<?php
/*******************************************************************************
*	This file is part of Woogle4MediaWiki
*   (http://www.mediawiki.org/wiki/Extension:Woogle4MediaWiki)
*
*	Copyright (c) 2007 - 2010 Hans-Jörg Happel and
*	FZI Forschungszentrum Informatik an der Universität Karlsruhe (TH)
*
*   Woogle4MediaWiki is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Woogle4MediaWiki is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Woogle4MediaWiki. If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/ 

// This is not a valid entry point, perform no further processing unless MEDIAWIKI is defined
if( !defined( 'MEDIAWIKI' ) ) {
	echo "This file is part of Woogle4MediaWiki and is not a valid entry point\n";
	die( 1 );
}

/**
 *	This file contains MediaWiki-specific utility functions
 */
class WoogleMWUtil {
	
	private static $db; 	// MediaWiki main database object (master9
	private static $db_slv; // MediaWiki main database object (slave/read only)
	
	/**
	 * Get the number of watches on a particular page in the Wiki
	 * 
	 * @param $title		string, page title
	 * @param $namespace	string, namespace id (e.g. '100')
	 * @return 				int, number of watches on this page
	 */
	public static function getNumWatches($title, $namespace=NS_MAIN) {
		WoogleUtil::profileIn(__METHOD__);

		// normalize to regular MW title
		$title = self::makeTitleDbKey($namespace, $title);
		
		// default return value
		$numWatchers = 0;
		
		// copied from Article.info()
		// TODO quoting?
		$dbr = self::getDB();			
		$wlClause = array(
			'wl_title' => $title,
			'wl_namespace' => $namespace 
		);
		$numWatchers = $dbr->selectField(
			'watchlist',
			'COUNT(*)',
			$wlClause,
			__METHOD__
		);

		WoogleUtil::profileOut(__METHOD__);
		return $numWatchers;
	}
	
	/**
	 * Retrieves all watched pages for a given namespace
	 * 
	 * @param $namespace		string, namespace id (e.g. '100')
	 * @return 					array of page titles (strings) each with a subarray of all userids
	 */
	public static function getWatches($namespace=null){
		WoogleUtil::profileIn(__METHOD__);
		
		$watches = array();
		
		$db = self::getDB();		

		extract($db->tableNames('watchlist'));
		$sql = 'SELECT wl_user, wl_title, wl_namespace FROM ' . $watchlist;
		if (is_numeric($namespace)){
			$namespace = intval($namespace);
			$sql .= ' WHERE wl_namespace = ' . $namespace;
		}

		$res = $db->doQuery($sql);
		
		while ($row = $db->fetchObject($res)) {
			// add user id to title
			$watches[$row->wl_title][] =  $row->wl_user;
			//$row->wl_namespace;
		}
		
		WoogleUtil::profileOut(__METHOD__);
		return $watches;
		
	}
	
	/**
	 *  Get the number of inlinks to a particular page in the Wiki
	 * 
	 * @param $title		string, page title
	 * @param $namespace	string, namespace id (e.g. '100')
	 * @return 				int, number of inlinks
	 */
	public static function getNumInlinks($title, $namespace=NS_MAIN) {
		WoogleUtil::profileIn(__METHOD__);
		
		// normalize to regular MW title
		$title = self::makeTitleDbKey($namespace, $title);
		
		// default return value
		$numPLinks = 0;
		
		// TODO quoting?
		
		$dbr = self::getDB();
		$plClause = array(
			'pl_title' 		=> $title,
			'pl_namespace' 	=> $namespace
		);
		$numPLinks = $dbr->selectField(
			'pagelinks',
			'COUNT(*)',
			$plClause,
			__METHOD__
		);

		WoogleUtil::debug('Found ' . $numPLinks . ' inlinks for page ' . $title . ' in namespaces ' . $namespace);
				
		WoogleUtil::profileOut(__METHOD__);
		return $numPLinks;
	}
	
	/**
	 * Singleton method for accessing MediaWiki db
	 * @param	$master		boolean, master or slave mode (=read only)
	 * @return 				Object, reference to MediaWiki db
	 */
	public static function getDB($master = true) {
		WoogleUtil::profileIn(__METHOD__);
		if (!isset(self::$db)) self::$db =& wfGetDB(DB_MASTER);
		if (!isset(self::$db_slv)) self::$db_slv =& wfGetDB(DB_SLAVE);
		WoogleUtil::profileOut(__METHOD__);
		if ($master){
			return self::$db;			
		} else {
			return self::$db_slv;
		}

	}
	
	/**
	 * Set an alternative database for the system
	 * This method is primarily intended to be called by test cases 
	 * 
	 * @param $extdb		Instance of a MediaWiki database object
	 * @param $master		boolean, master or slave mode (=read only)
	 * @return				void
	 */
	public static function setDB($extdb, $master = true){
		WoogleUtil::profileIn(__METHOD__);
		if ($master){
			self::$db = $extdb;
		} else {
			self::$db_slv = $extdb;
		}
		WoogleUtil::profileOut(__METHOD__);	
	}
	
	/**
	 * Send an E-Mail
	 * 
	 * @param $to			String, E-Mail of recipient
	 * @param $subject		String, E-Mail subject
	 * @param $body			String, E-Mail body
	 * @return				boolean, success
	 */
	public static function sendMail($to, $subject, $body){
		global $wgEnableEmail, $wgPasswordSender;
		WoogleUtil::profileIn(__METHOD__);
		
		$res = false;
		
		if ($wgEnableEmail){
			$to 	= new MailAddress($to);
			$from 	= new MailAddress($wgPasswordSender);
			$res 	= UserMailer::send($to,$from,$subject,$body); 
		}
			
		return WoogleUtil::profileOut(__METHOD__, $res);
	}
	
	
	//
	// Path methods 
	//
	
	/*
	 * @return 		String, Example: "http://localhost"
	 */
	public static function getAbsolutePath() {
		global $wgServer;
		return $wgServer;
	}

	/*
	 * @return 		String, Example: "http://localhost/wiki/"
	 */
	public static function getWikiAbsolutePath() {
		WoogleUtil::profileIn(__METHOD__);
		return WoogleUtil::profileOut(__METHOD__, self::getAbsolutePath() . self::getWikiPath());
	}

	/*
	 * Example $wgScriptPath = "/wiki"
	 * 
	 * @return 		String, Example "/wiki/"
	 */
	public static function getWikiPath() {
		global $wgScriptPath;
		WoogleUtil::profileIn(__METHOD__);
		return WoogleUtil::profileOut(__METHOD__, $wgScriptPath . '/');
	}	
	
	/*
	 * @return 		String, Example: "http://localhost/wiki/index.php/"
	 */
	public static function getArticleAbsolutePath() {
		WoogleUtil::profileIn(__METHOD__);
		return WoogleUtil::profileOut(__METHOD__, self::getAbsolutePath() . self::getArticlePath());
	}
			
	/*
	 * Example $wgArticlePath = "/wiki/index.php/$1";
	 * 
	 * @return 		String, Example: "/wiki/index.php/"
	 */
	public static function getArticlePath() {
		global $wgArticlePath;
		WoogleUtil::profileIn(__METHOD__);
		return WoogleUtil::profileOut(__METHOD__, str_replace('$1', '', $wgArticlePath));
	}
	
	/**
	 * 
	 * @param $wikiTime			MediaWiki concatenated string timestamp (YYYYMMDDHHMMSS) 
	 * @return 					SmallDateTime object
	 */
	public static function getDateTime($wikiTime) {
		// TODO - move - this is hessian-specific code!
		// TODO unit test
		
		WoogleUtil::profileIn(__METHOD__);
		$time = mktime(substr($wikiTime, 8, 2), substr($wikiTime, 10, 2), substr($wikiTime, 12, 2), substr($wikiTime, 4, 2),
			substr($wikiTime, 6, 2), substr($wikiTime, 0, 4));
		$dateTime = new SmallDateTime($time);
		WoogleUtil::profileOut(__METHOD__);
		return $dateTime;
	}
	
	/**
	 * 
	 * @param $time				SmallDateTime object
	 * 					$time = array(
	 * 						[day] => 25, 
     *                       [month] => 01 , 
     *                       [year] => 2010 ,
     *                       [hour] => 16 ,
     *                       [minute] => 49 , 
     *                       [second] => 28 ,
     *                    );
	 * 
	 * @return $timestamp		MediaWiki concatenated string timestamp (YYYYMMDDHHMMSS)
	 */
	public static function getWoogleTimestamp( $time ) {
		// TODO - move - this is hessian-specific code!
		WoogleUtil::profileIn(__METHOD__);
		/*
		$year = $time['year'];
		$month = $time['month'];
		$day = $time['day'];
		$hour = $time['hour'];
		$min = $time['minute'];
		$sec = $time['second'];
		
		$wooglets = $year.$month.$day.$hour.$min.$sec;
		*/

		$wooglets = '0';
		// TODO avoid this check!
		if (method_exists($time, 'strftime')){
			$wooglets = $time->strftime('YmdGis'); // c.f. http://de2.php.net/manual/en/function.date.php
		} 
		
		WoogleUtil::profileOut(__METHOD__);
		return $wooglets;
	}
	
	
	/**
	 * check if certain privilege on db is granted for current user
	 * @param $action			db priviliges to check (e.g. 'DROP')
	 * @return boolean			yes or no	
	 */
	public static function checkDbPrivileges($action){
		WoogleUtil::profileIn(__METHOD__);
		
		$db =& wfGetDB(DB_SLAVE);
		
		// query to get all rights for current db user on db
		$res = $db->query( "SHOW GRANTS FOR CURRENT_USER;" );
		
		for ($i=0; $i<$db->numRows($res); $i++){
			$row = $db->fetchRow( $res );
			
			// check if right applies to mediawiki db or all dbs
			if (strpos($row[0], $db->getDBname()) || (strpos($row[0], 'ON *.*')) ){

				// check for universal priviliges
				if (strpos($row[0], 'ALL PRIVILEGES')) return true;

				// remove rights which are only temporary
				$str_check = str_replace($action . ' TEMPORARY', 'VOID', $row[0]);	

				// check for special priviliges
				if (strpos($str_check, $action)) return true;
				
			}
			WoogleUtil::debug('Check priviliges content: ' . $row[0]);
		}
		
		WoogleUtil::profileOut(__METHOD__);
		return false;
	}
	
		
	/**
	 * 
	 * @param $email			String, E-Mail address to validate
	 * @return 					boolean
	 */
	public static function isEmailValid( $email ){
		WoogleUtil::profileIn(__METHOD__);
		$res = false;
		if ( filter_var($email, FILTER_VALIDATE_EMAIL) != false ) $res = true;
		WoogleUtil::profileOut(__METHOD__);
		return $res; 
	}	
	
	/**
	 * Returns HTML-output for a MediaWiki article
	 * 
	 * @param $article			MW.article
	 * @param $striptags		boolean, remove HTML tags
	 * @return					String, article text
	 */
	public static function parseArticle($article, $striptags = true){
		global $wgParser, $wgOut;
		WoogleUtil::profileIn(__METHOD__);
		$parseroutput = $wgParser->parse($article->getContent(), $article->getTitle(), $wgOut->parserOptions());
		$content = $parseroutput->getText(); 
		if ($striptags) $content = strip_tags( $content );
		WoogleUtil::profileOut(__METHOD__);
		return $content;
	}
	
	/**
	 * Returns HTML-output for a wiki page title
	 * 
	 * @param $article			title of MediaWiki page
	 * @param $striptags		boolean, remove HTML tags
	 * @return					String, article text
	 */
	public static function parseArticleFromTitle($title, $striptags = true){
		WoogleUtil::profileIn(__METHOD__);
		$t = new Title();
		$tt = $t->newFromDBkey($title);
		$article = new Article($tt);
		$res = self::parseArticle($article, $striptags);
		WoogleUtil::profileOut(__METHOD__);
		return $res;
	}
	
	/**
	 * Checks if a certain table exists in the MediaWiki db
	 * @param $tablename	string, name of database table
	 * @return 				boolean, if $tablename exsits
	 */
	public static function tableExists($tablename){
		WoogleUtil::profileIn(__METHOD__);
		$db = WoogleMWUtil::getDB(false);
		$res = $db->tableExists($tablename);
		WoogleUtil::profileOut(__METHOD__);
		return $res;
	}
	
	/**
	 * Removes MediaWiki markup from a text
	 * 
	 * @param $content			String, MediaWiki content containing MediaWiki markup
	 * @return 					Strgin, content without markup
	 */
	public static function stripMediaWikiMarkup($content){
		WoogleUtil::profileIn(__METHOD__);
		$content = strip_tags($content);						// HTML
		$content = str_replace('--', '', $content); 			// double dash
		$content = str_replace('~', '', $content); 				// tilde
		$content = str_replace('=', ' ', $content); 			// equals
		$content = str_replace('[', '', $content); 				// bracket start
		$content = str_replace(']', '', $content); 				// bracket stop
		$content = str_replace('*', '', $content); 				// asterisk
		$content = str_replace('#', '', $content); 				// hash
		$content = str_replace('\'', '', $content); 			// tick
		$content = preg_replace('/__.*__/', '', $content);		// magic words
		$content = preg_replace('/\{\{.*\}\}/', '', $content);	// variables/templates

		// TODO Table {CONTENT}
		$content = str_replace('{', '', $content); 				// remaining {
		$content = str_replace('}', '', $content); 				// remaining }
		$content = str_replace('|', '', $content); 				// pipe
		
		$userDefined = WoogleConfig::get('stripMWMarkup');
		foreach($userDefined as $pair){
			WoogleUtil::debug('Applying user-defined regular expression: ' . $pair[0] . ' => ' . $pair[1]);
			$content = preg_replace($pair[0], $pair[1]);	
		}	
		
		WoogleUtil::profileOut(__METHOD__);
		return $content;
	}
	
	public static function endsWith( $str, $sub ) {
		WoogleUtil::profileIn(__METHOD__);
		$res = ( substr( $str, strlen( $str ) - strlen( $sub ) ) === $sub );
 		WoogleUtil::profileOut(__METHOD__);
		return $res;
	}
	
	/**
	 * Normalizes a Wiki page title to a valid MediaWiki DbKey (e.g. uppercase first letter) 
	 * 
	 * @param	$namespace	String, namespace id
	 * @param 	$title		String, wiki page title
	 * @param	$prefix		boolean, if to include namespace
	 * @return				String, valid MediaWiki DbKey
	 */
	public static function makeTitleDbKey($namespace, $title, $prefix=false){
		WoogleUtil::profileIn(__METHOD__);
		$title = trim($title);
		$mwTitle = Title::makeTitle(intval($namespace), $title);
		$res = $mwTitle->getDBkey();
		
		// capitalize first char
		// this is not done by makeTitle! - Title::capitalize() introduced only in later MW versions
		global $wgCapitalLinks, $wgContLang;
		if( $wgCapitalLinks ) {
			$res = $wgContLang->ucfirst( $res );
		}
		
		if ($prefix) $res = $mwTitle->getPrefixedDBkey();
		WoogleUtil::profileOut(__METHOD__);
		return $res;
		
	}
	
}


?>